/*
Very simple OpenCV program.

Pixel access discussed here:
  http://stackoverflow.com/questions/4742251/pixel-access-in-opencv-2-2

cv::Mat described here:
  http://docs.opencv.org/2.4/modules/core/doc/core.html


Dr. Orion Lawlor, lawlor@alaska.edu, 2015-10-30 (Public Domain)
*/
#include <stdio.h>

#include <opencv2/opencv.hpp>


int main(int argc,char *argv[])
{
	cv::VideoCapture *cap=0;
	
	int argi=1; // command line argument index
	while (argc>=argi+2) { /* keyword-value pairs */
		char *key=argv[argi++];
		char *value=argv[argi++];
		if (0==strcmp(key,"--cam")) cap=new cv::VideoCapture(atoi(value));
		if (0==strcmp(key,"--file")) cap=new cv::VideoCapture(value);
		else printf("Unrecognized command line argument '%s'!\n",key);
	}
	// Initialize capturing live feed from the camera
	if (!cap) cap=new cv::VideoCapture(0);
	
	// Couldn't get a device? Throw an error and quit
	if(!cap->isOpened())
	{
		printf("Could not initialize capturing...\n");
		return -1;
	}
	
	// Will hold a frame captured from the camera
	cv::Mat frame;

	// An infinite frame-capturing loop
	while(true)
	{
		(*cap)>>frame; // grab next frame from camera

		// If we couldn't grab a frame... quit
		if(frame.empty())
			break;
		
		// Manually access OpenCV pixels, stored in BGR order
		typedef cv::Vec3b pixel;
		
		double sum_x=0, sum_y=0, sum_pixels=0;
		for (int y=0;y<frame.rows;y++) 
		for (int x=0;x<frame.cols;x++){
			pixel p=frame.at<pixel>(y,x);
			int r=p[2], g=p[1], b=p[0]; // BGR order
			// Look for bright red pixels
			if (r>g+b  && r>200) {
				// std::cout<<x<<","<<y<<": "<<r<<","<<g<<","<<b<<"\n";
				sum_x+=x; sum_y+=y; sum_pixels++;
			}
		}
		
		// Compute center of mass
		double avg_x=sum_x/sum_pixels; // weighted average
		double avg_y=sum_y/sum_pixels;
		
		if (sum_pixels>0) 
			std::cout<<"Saw "<<sum_pixels<<" red pixels COM: "<<avg_x<<", "<<avg_y<<"\n";
		
		// Draw crosshairs
		float cross=20;
		cv::Scalar crossColor(255,255,255);
		cv::line(frame,cv::Point(avg_x,avg_y+cross), cv::Point(avg_x,avg_y-cross), crossColor);
		cv::line(frame,cv::Point(avg_x+cross,avg_y), cv::Point(avg_x-cross,avg_y), crossColor);
		
		
		
		// Show the video frame:
		cv::imshow("video", frame);

		// Wait for a keypress (for up to 1ms)
		int c = cv::waitKey(1);
		if(c!=-1)
		{
			// If any key is pressed, break out of the loop.
			//   MUST exit cleanly to avoid killing linux webcams.
			break;
		}
	}

	return 0;
}
